import {
    CollectionViewer,
    DataSource
} from '@angular/cdk/collections';

import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { of } from 'rxjs/observable/of';

import {
    catchError,
    finalize
} from 'rxjs/operators';

import { GiftedSubsService } from './giftedsubs.service';

import { GiftedSub } from '../models/giftedsub';


export class GiftedSubsDataSource implements DataSource<GiftedSub> {
    private giftedSubsSubject = new BehaviorSubject<GiftedSub[]>([]);
    private countSubject = new BehaviorSubject<number>(0);
    private loadingSubject = new BehaviorSubject<boolean>(false);

    public count$ = this.countSubject.asObservable();
    public loading$ = this.loadingSubject.asObservable();

    constructor(private giftedSubsService: GiftedSubsService) { }

    loadGiftedSubs(filter: string,
                   sortColumn: string,
                   sortDirection: string,
                   pageIndex: number,
                   pageSize: number) {

        this.loadingSubject.next(true);

        this.giftedSubsService.findGiftedSubs(filter, sortColumn, sortDirection,
                                              pageIndex, pageSize)
            .pipe(
                catchError(() => of([])),
                finalize(() => this.loadingSubject.next(false))
            )
            .subscribe(data => {
                this.giftedSubsSubject.next(data.giftedSubs);
                this.countSubject.next(data.totalCount);
            });
    }

    connect(collectionViewer: CollectionViewer): Observable<GiftedSub[]> {
        console.log('Connecting data source');
        return this.giftedSubsSubject.asObservable();
    }

    disconnect(collectionViewer: CollectionViewer): void {
        this.giftedSubsSubject.complete();
        this.countSubject.complete();
        this.loadingSubject.complete();
    }
}
