import {
    AfterViewInit,
    Component,
    ElementRef,
    OnInit,
    ViewChild
} from '@angular/core';

import { ActivatedRoute } from '@angular/router';

import {
    MatPaginator,
    MatSort,
    MatTableDataSource
} from '@angular/material';

import { DatePipe } from '@angular/common';

import {
    debounceTime,
    distinctUntilChanged,
    startWith,
    tap,
    delay
} from 'rxjs/operators';

import { merge } from 'rxjs/observable/merge';
import { fromEvent } from 'rxjs/observable/fromEvent';

import { RegularSubsService } from '../services/regularsubs.service';
import { RegularSubsDataSource } from '../services/regularsubs.datasource';


@Component({
    selector: 'regular-subs',
    templateUrl: './regularsubs.component.html',
    styleUrls: [ './regularsubs.component.css' ]
})
export class RegularSubsComponent implements OnInit, AfterViewInit {
    dataSource: RegularSubsDataSource;

    displayedColumns = [
        'time',
        'user',
        'rank'
    ];

    @ViewChild(MatPaginator) paginator: MatPaginator;
    @ViewChild(MatSort) sort: MatSort;
    @ViewChild('input') input: ElementRef;

    constructor(private route: ActivatedRoute,
                private regularSubsService: RegularSubsService) { }

    ngOnInit() {
        this.dataSource = new RegularSubsDataSource(this.regularSubsService);
        this.dataSource.loadRegularSubs('', 'time', 'desc', 0, 20);
    }

    ngAfterViewInit() {
        this.sort.sortChange.subscribe(() => this.paginator.pageIndex = 0);

        fromEvent(this.input.nativeElement, 'keyup')
            .pipe(
                debounceTime(150),
                distinctUntilChanged(),
                tap(() => {
                    this.paginator.pageIndex = 0;
                    this.loadRegularSubsPage();
                })
            )
            .subscribe();

        merge(this.sort.sortChange, this.paginator.page)
            .pipe(
                tap(() => this.loadRegularSubsPage())
            )
            .subscribe();
    }

    loadRegularSubsPage() {
        this.dataSource.loadRegularSubs(
            this.input.nativeElement.value,
            this.sort.active,
            this.sort.direction,
            this.paginator.pageIndex,
            this.paginator.pageSize);
    }
}
